rem *********************************************************
rem  AUCTION.DDL 
rem
rem  Creates objects used in the "online auction" example
rem  for the DBMS_ALERT package.  
rem
rem  Auction_items --  table of items being auctioned
rem  Bids -- table of bids placed on items
rem  High_bids -- view showing the current high bids on
rem               items and who placed them
rem
rem  Author:  John Beresniewicz, Savant Corp
rem
rem
rem  12/07/97: created
rem *********************************************************
DROP VIEW high_bids;
DROP TABLE bids;
DROP TABLE auction_items;

CREATE TABLE auction_items 
   (id            VARCHAR2(20)  NOT NULL PRIMARY KEY
   ,description   VARCHAR2(200) NOT NULL
   ,min_bid       NUMBER        NOT NULL
   ,curr_bid      NUMBER
   ,status        VARCHAR2(10) 
                  CONSTRAINT   valid_status
                  CHECK (status IN ('OPEN','CLOSED') )
   );

CREATE TABLE bids
   (bidder        VARCHAR2(30)
   ,item_id       VARCHAR2(20)  
                  REFERENCES auction_items(id) 
                  ON DELETE CASCADE
   ,bid           NUMBER         NOT NULL
   );

CREATE OR REPLACE VIEW high_bids
   (item_id
   ,item_desc
   ,bidder
   ,high_bid)
AS
SELECT
        BID.item_id
       ,AI.description
       ,BID.bidder
       ,BID.bid
  FROM
        bids           BID
       ,auction_items  AI
 WHERE
        BID.item_id    = AI.id
   AND  BID.bid        = (SELECT  MAX(bid) 
                            FROM  bids    B2
                           WHERE  BID.item_id = B2.item_id)
/

