CREATE OR REPLACE TRIGGER auction_items_ARU
AFTER UPDATE ON auction_items
FOR EACH ROW
BEGIN
   /* 
   || trigger enforces no update of item_id and also
   || signals an alert when status changes
   */
   IF UPDATING ('ITEM_ID')
   THEN 
      RAISE_APPLICATION_ERROR(-20000, 'Cannot update item id');
   
   ELSIF UPDATING ('STATUS') AND (:NEW.status != :OLD.status)
   THEN
      /* send new status on as the alert message */
      DBMS_ALERT.SIGNAL(:NEW.id, :NEW.status);
   END IF;
END auction_items_ARU;
/

CREATE OR REPLACE TRIGGER bids_ARIUD
AFTER INSERT OR UPDATE OR DELETE ON bids
FOR EACH ROW
BEGIN
   /* 
   || enforce all bids are final rule
   */
   IF UPDATING OR DELETING
   THEN
      RAISE_APPLICATION_ERROR
         (-20001, 'Cannot update or delete, all bids final!');
   ELSE
   /*
   || signal alert on item, send bidder name as message
   */
   DBMS_ALERT.SIGNAL(:NEW.item_id, :NEW.bidder);
   END IF;

END bids_ARIUD;
/

