SET FEEDBACK OFF
SET VERIFY OFF
DECLARE
/*
|| Approach 1: the worst
*/
   v_start INTEGER;
   cursor_id INTEGER;
   exec_stat INTEGER;

BEGIN
   v_start := DBMS_UTILITY.GET_TIME;
   cursor_id := DBMS_SQL.OPEN_CURSOR;
   FOR i IN 1..&1
   LOOP
      /* 
      || parse and excecute each loop iteration
      || without using host vars, this is worst case
      */
      DBMS_SQL.PARSE
         (cursor_id,
          'SELECT employee_id, last_name, first_name, d.department_id, ' ||
          '       salary, manager_id' ||
          '  FROM employee E, department D ' || 
          ' WHERE D.department_id = ' || 30 || 
          '   AND E.department_id = D.department_id' ||
          '   AND E.employee_id IN ' ||
          '       (SELECT employee_id ' ||
          '          FROM employee E2' ||
          '         WHERE salary = (SELECT MAX(salary) FROM employee))' ||
          '   AND last_name LIKE ''S%''' ||
          ' ORDER BY DECODE (D.department_id, 10, SYSDATE, ' ||
          '    20, SYSDATE + 50,' ||
          '    30, ADD_MONTHS (SYSDATE,50),' ||
          '    40, SYSDATE + 500)'
         ,DBMS_SQL.native);
      exec_stat := DBMS_SQL.EXECUTE(cursor_id);
   END LOOP;
   DBMS_SQL.CLOSE_CURSOR(cursor_id);
   DBMS_OUTPUT.PUT_LINE 
      ('Approach 1: ' || TO_CHAR (DBMS_UTILITY.GET_TIME - v_start));
END;
/

DECLARE
/*
|| Approach 2: a little better
*/
   v_start INTEGER;
   cursor_id INTEGER;
   exec_stat INTEGER;
BEGIN
   v_start := DBMS_UTILITY.GET_TIME;
   cursor_id := DBMS_SQL.OPEN_CURSOR;
   FOR i IN 1..&1
   LOOP
      /* 
      || parse and excecute each loop iteration
      || using host vars
      */
      DBMS_SQL.PARSE
         (cursor_id,
          'SELECT employee_id, last_name, first_name, d.department_id, ' ||
          '       salary, manager_id' ||
          '  FROM employee E, department D ' || 
          ' WHERE D.department_id = :i' || 
          '   AND E.department_id = D.department_id' ||
          '   AND E.employee_id IN ' ||
          '       (SELECT employee_id ' ||
          '          FROM employee E2' ||
          '         WHERE salary = (SELECT MAX(salary) FROM employee))' ||
          '   AND last_name LIKE ''S%''' ||
          ' ORDER BY DECODE (D.department_id, 10, SYSDATE, ' ||
          '    20, SYSDATE + 50,' ||
          '    30, ADD_MONTHS (SYSDATE,50),' ||
          '    40, SYSDATE + 500)'
         ,DBMS_SQL.native);
      DBMS_SQL.BIND_VARIABLE(cursor_id,'i',i);
      exec_stat := DBMS_SQL.EXECUTE(cursor_id);
   END LOOP;
   DBMS_SQL.CLOSE_CURSOR(cursor_id);
   DBMS_OUTPUT.PUT_LINE 
      ('Approach 2: ' || TO_CHAR (DBMS_UTILITY.GET_TIME - v_start));
END;
/

DECLARE
/*
|| Approach 3: the best
*/
   v_start INTEGER;
   cursor_id INTEGER;
   exec_stat INTEGER;
BEGIN
   v_start := DBMS_UTILITY.GET_TIME;
   cursor_id := DBMS_SQL.OPEN_CURSOR;
   /*
   || parse first outside of loop
   */
   DBMS_SQL.PARSE
      (cursor_id,
          'SELECT employee_id, last_name, first_name, d.department_id, ' ||
          '       salary, manager_id' ||
          '  FROM employee E, department D ' || 
          ' WHERE D.department_id = :i' || 
          '   AND E.department_id = D.department_id' ||
          '   AND E.employee_id IN ' ||
          '       (SELECT employee_id ' ||
          '          FROM employee E2' ||
          '         WHERE salary = (SELECT MAX(salary) FROM employee))' ||
          '   AND last_name LIKE ''S%''' ||
          ' ORDER BY DECODE (D.department_id, 10, SYSDATE, ' ||
          '    20, SYSDATE + 50,' ||
          '    30, ADD_MONTHS (SYSDATE,50),' ||
          '    40, SYSDATE + 500)'
      ,DBMS_SQL.native);
   FOR i IN 1..&1
   LOOP
      /* 
      || bind and excecute each loop iteration
      || using host vars
      */
      DBMS_SQL.BIND_VARIABLE(cursor_id,'i',i);
      exec_stat := DBMS_SQL.EXECUTE(cursor_id);
   END LOOP;
   DBMS_SQL.CLOSE_CURSOR(cursor_id);
   DBMS_OUTPUT.PUT_LINE 
      ('Approach 3: ' || TO_CHAR (DBMS_UTILITY.GET_TIME - v_start));
END;
/
