CREATE OR REPLACE
PROCEDURE break_all_jobs (set_broken_IN IN BOOLEAN)
IS
   /*
   || Sets the broken flag to TRUE or FALSE for all 
   || jobs in the job queue
   ||
   || Requirements:
   ||
   || SELECT on DBA_JOBS
   || EXECUTE on DBMS_IJOB
   */
BEGIN
   FOR job_rec IN 
            (SELECT job 
               FROM dba_jobs)
   LOOP
      SYS.DBMS_IJOB.BROKEN(job_rec.job, set_broken_IN);
   END LOOP;
END break_all_jobs;
/

CREATE OR REPLACE
PROCEDURE remove_all_jobs
   (owner_IN IN VARCHAR2 := USER )
IS
   /*
   || Removes all jobs from the job queue owned by
   || a specific user, defaults to current user
   ||
   || Requirements:
   ||
   || SELECT on DBA_JOBS
   || EXECUTE on DBMS_IJOB
   */
BEGIN
   FOR job_rec IN 
            (SELECT job 
               FROM dba_jobs 
              WHERE priv_user = NVL(owner_IN,priv_user) )
   LOOP
      SYS.DBMS_IJOB.REMOVE(job_rec.job);
   END LOOP;
END remove_all_jobs;
/
