DECLARE
   total_blocks  NUMBER;
   total_bytes   NUMBER;
   unused_blocks NUMBER;
   unused_bytes  NUMBER;
   last_extent_file  NUMBER;
   last_extent_block NUMBER;
   last_block    NUMBER;
   grand_total_blocks NUMBER := 0;
   grand_total_unused NUMBER := 0;

BEGIN
   FOR user_tables_rec IN
      (SELECT table_name 
         FROM user_tables)
   LOOP
      DBMS_SPACE.UNUSED_SPACE
         (segment_owner => USER
         ,segment_name  => user_tables_rec.table_name
         ,segment_type  => 'TABLE'
         ,total_blocks  => total_blocks
         ,total_bytes   => total_bytes
         ,unused_blocks => unused_blocks
         ,unused_bytes  => unused_bytes
         ,last_used_extent_file_id => last_extent_file
         ,last_used_extent_block_id => last_extent_block
         ,last_used_block  => last_block
         );

      grand_total_blocks := grand_total_blocks + total_blocks;
      grand_total_unused := grand_total_unused + unused_blocks;
   END LOOP;

   DBMS_OUTPUT.PUT_LINE('Space utilization (TABLES) ');
   DBMS_OUTPUT.PUT_LINE('total blocks: '||TO_CHAR(grand_total_blocks) );
   DBMS_OUTPUT.PUT_LINE('unused blocks: '||TO_CHAR(grand_total_unused) );
   DBMS_OUTPUT.PUT_LINE('pct unused: '||
      TO_CHAR(ROUND((grand_total_unused/grand_total_blocks)*100) ) );
END;
/


DECLARE
   grand_total_blocks NUMBER := 0;
   grand_total_unused NUMBER := 0;

BEGIN
   FOR user_tables_rec IN
      (SELECT table_name 
         FROM user_tables)
   LOOP
      grand_total_blocks := grand_total_blocks + 
                              segspace.total_blocks(user_tables_rec.table_name);

      grand_total_unused := grand_total_unused + 
                              segspace.unused_blocks(user_tables_rec.table_name);
   END LOOP;

   DBMS_OUTPUT.PUT_LINE('Space utilization (TABLES) ');
   DBMS_OUTPUT.PUT_LINE('total blocks: '||TO_CHAR(grand_total_blocks) );
   DBMS_OUTPUT.PUT_LINE('unused blocks: '||TO_CHAR(grand_total_unused) );
   DBMS_OUTPUT.PUT_LINE('pct unused: '||
      TO_CHAR(ROUND((grand_total_unused/grand_total_blocks)*100) ) );
END;
/

DECLARE
   free_blocks   NUMBER;
BEGIN
   DBMS_OUTPUT.PUT_LINE(RPAD('TABLE NAME',30)||' FREELIST BLOCKS');

   FOR user_tables_rec IN
      (SELECT table_name 
         FROM user_tables)
   LOOP
      DBMS_SPACE.FREE_BLOCKS
         (segment_owner => USER
         ,segment_name  => user_tables_rec.table_name
         ,segment_type  => 'TABLE'
         ,freelist_group_id  => 0
         ,free_blks => free_blocks
         ,scan_limit => NULL);
      
      DBMS_OUTPUT.PUT_LINE(RPAD(user_tables_rec.table_name,30)||' '||
                  TO_CHAR(free_blocks));
   END LOOP;
END;
/


